/***************************************************************************/
/*                                                                         */
/*        FileName:     CMp3Header.cc                                      */
/*        Created By:   Owen Lloyd                                         */
/*                                                                         */
/*        Date:         November 3rd, 2000                                 */
/*        Email:        o_lloyd@hotmail.com                                */
/*                                                                         */
/*        Compatible:   Linux, Windows                                     */
/*                                                                         */
/*        Summary:      This class will extract the .mp3 frame header      */
/*                      information a mp3 file.                            */
/*                                                                         */
/***************************************************************************/

#include <string.h>
#include "CMp3Header.h"

CMp3Header::CMp3Header()
{
	Initialize();
	bValid = false;
}

CMp3Header::CMp3Header(char *cpName)
{
	Initialize();
	strcpy (cFileName, cpName);
	bValid = (bLoadData() && bValid) ? true : false;

	if (bValid)
	{
		GetData(1,11,iSyncWord);
		GetData(12,2,iMPEGVersion);
		GetData(14,2,iMPEGLayer);
		GetData(16,1,iProtection);
		GetData(17,4,iBitRate);
		GetData(21,2,iSampleFreq);
		GetData(22,1,iPaddingBit);
		GetData(23,1,iPrivateBit);
		GetData(25,2,iMode);
		GetData(27,2,iModeExt);
		GetData(29,1,iCopyright);
		GetData(30,1,iOrigHome);
		GetData(31,2,iEmphasis);

		bValid = (iGetBitRate() && bValid) ? true : false;
		bValid = (iGetSampleFreq() && bValid) ? true : false;
		bValid = (iGetTime() && bValid) ? true : false;
	}
}

void CMp3Header::Initialize()
{

	bValid        = true;
	iSyncWord     = 0;
	iMPEGVersion  = 0;
	iMPEGLayer    = 0;
	iProtection   = 0;
	iBitRate      = 0;
	iBitRateReal  = 0;
	iSampleFreq   = 0;
	iPaddingBit   = 0;
	iPrivateBit   = 0;
	iMode         = 0;
	iModeExt      = 0;
	iCopyright    = 0;
	iOrigHome     = 0;
	iEmphasis     = 0;
	iFileSize     = 0;
	iTime         = 0;

}

void CMp3Header::ShowResults()
{
	char cBuffer[128];
	char cTmp[1024];

	sprintf (cBuffer, "\n%s - File Data\n", cpExtractFileName(&cTmp[0]));
	printf (cBuffer);
	for (int i=0; (i<(signed)strlen(cBuffer)-2) && (i<80); i++) printf("-");
	printf ("\n");
	
	if (!bValid)
	{
		printf ("Invalid MP3\n");
		return;
	}
	
	printf ("File Size: \t%d bytes\n", iFileSize);
	printf ("Est. Time: \t%d secs\n", iGetTime());
	printf ("MPEG Version: \t%d\n", iGetMPEGVersion());
	
	printf ("MPEG Layer: \t");
	for (int i=iGetMPEGLayer(); i>0; i--) printf("I");
	printf ("\n");
	
	printf ("BitRate: \t%d kBit/s\n", iGetBitRate());
	printf ("Sample Freq: \t%d kHz\n", iGetSampleFreq());
	printf ("Padding: \t%s\n", (iGetPadding())? "Yes" : "No");
	printf ("Mode: \t\t");
	switch (iGetMode())
	{
	case 0:
		printf ("Stereo\n");
		break;
	case 1:
		printf ("Joint Stereo\n");
		break;
	case 2:
		printf ("Dual Channel\n");
		break;
	case 3:
		printf ("Single Channel\n");
		break;
	default:
		printf ("\n");
		break;
	}
	printf ("Private: \t%s\n", (iGetPrivate()) ? "Yes" : "No");
	printf ("Copyright: \t%s\n", (iGetCopyright()) ? "Yes" : "No");
	printf ("Orginal: \t%s\n", (iGetOriginal()) ? "Yes" : "No");
	printf ("Emphasis: \t");
	switch (iGetEmphasis())
	{
	case 0:
		printf ("None\n");
		break;
	case 1:
		printf ("50/15 ms\n");
		break;
	case 2:
		printf ("reserved\n");
		break;
	case 3:
		printf ("CCIT J.17\n");
		break;
	default:
		printf ("\n");
		break;
	}
	
	printf ("\n");


}

bool CMp3Header::bLoadData()
{
	FILE *FDATA = fopen(cFileName,"r");
	bool bTest  = false;
	int  iSize  = 0;

	iData = 0;

	fseek(FDATA,0,SEEK_END);  // move to end of file and record the size
	iFileSize = ftell(FDATA);
	fseek(FDATA,0,SEEK_SET);
	
	//load the first 32 bits of the file
	while (!bTest)
	{
		iData = 0;
		iData ^= (unsigned)fgetc(FDATA);
		if (++iSize > iFileSize) break;	
		if (!((iData & 0xFF)==0xFF)) continue;     //invalid frame sync
		iData <<= 8;
		iData ^= (unsigned)fgetc(FDATA);
		if (++iSize > iFileSize) break;	
		if (!((iData & 0xFFE0)==0xFFE0)) continue; //invalid frame sync
		
		// framse sync should be valid
		bTest = true;

		for (int i=2; i < 4; i++) 
		{
			iData <<= 8;
			iData ^= (unsigned)fgetc(FDATA);
		}
	}

	fclose (FDATA);
	
	return bTest;
}

void CMp3Header::GetData(int iLocation, int iWidth, int &iVar)
{
	unsigned int iTmp=iData;
	unsigned int uiMask=0;
	int iCount = iWidth;

	while (iCount)
	{
		uiMask<<=1;
		uiMask^=0x1;
		iCount--;
	}

	iTmp >>= 32 - iLocation - iWidth + 1;
	iTmp  &= uiMask;

	iVar   = iTmp;
}

int CMp3Header::iGetBitRate()
{
	int a[16][5] =	{
		
		{     0,   0,   0,   0,   0},  //0
		{    32,  32,  32,  32,   8},  //1
		{	 64,  48,  40,  48,  16},  //2
		{	 96,  56,  48,  56,  24},  //3
		{	128,  64,  56,  64,  32},  //4
		{	160,  80,  64,  80,  40},  //5
		{	192,  96,  80,  96,  48},  //6
		{	224, 112,  96, 112,  56},  //7
		{	256, 128, 112, 128,  64},  //8
		{	288, 160, 128, 144,  80},  //9
		{	320, 192, 160, 160,  96},  //A
		{	352, 224, 192, 176, 112},  //B
		{	384, 256, 224, 192, 128},  //C
		{	416, 320, 256, 224, 144},  //D
		{	448, 384, 320, 256, 160},  //E
		{     0,   0,   0,   0,   0},  //F
	};

	int y=0;

	if (iMPEGVersion==3 && iMPEGLayer==3) y=0;        //V1, L1
	else if (iMPEGVersion==3 && iMPEGLayer==2) y=1;   //V1, L2
	else if (iMPEGVersion==3 && iMPEGLayer==1) y=2;   //V1, L3
	else if (iMPEGVersion==2 && iMPEGLayer==3) y=3;   //V2, L1
	else return 0;

	iBitRateReal = a[iBitRate][y];

	return iBitRateReal;
}

int CMp3Header::iGetSampleFreq()
{
	int a[4][2] = { {44100,22050},
					{48000,24000},
					{32000,16000},
					{    0,    0}
	};

	return a[iSampleFreq][(iMPEGVersion==3) ? 0 : 1];
}

int CMp3Header::iGetFileSize()
{
	return iFileSize;
}

int CMp3Header::iGetTime()
{
	if (iBitRateReal==0) return 0;
	
	return iTime = iFileSize / (iBitRateReal*1000/8);
}

int CMp3Header::iGetMPEGVersion()
{
	if (iMPEGVersion==2) return 2;      // MPEG 2
	else if (iMPEGVersion==3) return 1; // MPEG 1
	
	return 0;
}

int CMp3Header::iGetMPEGLayer()
{
	if (iMPEGLayer==1) return 3;      // Layer 3
	else if(iMPEGLayer==2) return 2;  // Layer 2
	else if(iMPEGLayer==3) return 1;  // Layer 1
	
	return 0;
}

int CMp3Header::iGetPadding()
{
	return iPaddingBit;
}

int CMp3Header::iGetMode()
{
	return iMode;  // 0 = stereo
	           // 1 = joint stereo
	           // 2 = dual channel
	           // 3 = single channel
}

int CMp3Header::iGetModeExt()
{
	return iModeExt;
}

int CMp3Header::iGetCRC()
{
	return 0;
}

int CMp3Header::iGetPrivate()
{
	return iPrivateBit;
}

int CMp3Header::iGetCopyright()
{
	return iCopyright;  // 0=no, 1=copyrighted
}

int CMp3Header::iGetOriginal()
{
	return iOrigHome;  // 0=copy, 1=original media
}

int CMp3Header::iGetEmphasis()
{
	return iEmphasis;  // 0 = none
	                   // 1 = 50/15 ms
	                   // 2 = reserved
	                   // 3 = CCIT J.17
}

char * CMp3Header::cpExtractFileName(char cBuffer[])
{
	char *cpTmp  = cFileName;	
	char *cpGood = cFileName;

	while (*cpTmp != '\0')
	{
		if (*cpTmp == '/')
		{
			cpGood = cpTmp;
		}
		*cpTmp++;
	}
	if (*cpGood == '/') *cpGood++;

	strcpy (cBuffer, cpGood);

	return cpGood;
}

bool CMp3Header::bGetValid()
{
	return bValid;
}
